//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

double Lots                = 0.1;
double MaximumRisk         = 0.02;
double DecreaseFactor      = 3;
extern double MAPer_Low    = 12;
extern double MAPer_Fast   = 12;
extern int PRICE           = 1;     // PRICE_CLOSE 0; PRICE_OPEN 1; PRICE_HIGH 2; PRICE_LOW 3;
                                    // PRICE_MEDIAN 4  , (high+low)/2; PRICE_TYPICAL 5  , (high+low+close)/3;
                                    // PRICE_WEIGHTED 6   , (high+low+close+close)/4;
extern int Real_NoReal     = 1;     // 0-   ,     " ", 
                                    //   "  ",  PRICE    1( ) 
                                    // 1 -  "  "( O=H=L=C,     )                                    
extern int MODE_MA         = 0;     // MODE_SMA 0   ; MODE_EMA 1   ;  ..                               
//extern double Mo__vingShift        = 6;
int OnStart = 1;

//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   //return(lot);
   return(1.0);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma_low, ma_fast, ma_low.1, ma_fast.1;
   int    res;
//---- go trading only for first tiks of new bar
   //if(Volume[0]>1) return;
   
//----
  int Ticket, Tip;
  double Lts;
  Ticket=GetTicketLastPos("", -1, MAGICMA);
    
  if (Ticket>0) 
      {OrderSelect(Ticket, SELECT_BY_TICKET);
       Tip   = OrderType();                                
       Lts   = OrderLots();}   
//---- get Moving Average 
   ma_low=iMA(NULL,0,MAPer_Low,0,MODE_MA,PRICE,Real_NoReal); // 
   ma_fast=iMA(NULL,0,MAPer_Fast,0,MODE_MA,PRICE,Real_NoReal); // 
   ma_low.1=iMA(NULL,0,MAPer_Low,0,MODE_MA,PRICE,Real_NoReal+1); // 
   ma_fast.1=iMA(NULL,0,MAPer_Fast,0,MODE_MA,PRICE,Real_NoReal+1); // 
   
//---- sell conditions
   if(ma_fast < ma_low && ma_fast.1 > ma_low.1)  
     {
      if (Ticket>0 && Tip == 0) 
        if (OrderClose(Ticket,Lts,Bid,3,White))
             res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
//---- buy conditions
   if(ma_fast > ma_low && ma_fast.1 < ma_low.1)  
     {
      if (Ticket>0  && Tip == 1) 
        if (OrderClose(Ticket,Lts,Ask,3,White))
           res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }
//----
  }

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
   if (OnStart==1)
     {
      OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      OnStart = 2;
     }
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
   
//---- calculate open orders by current symbol
   //if(CalculateCurrentOrders(Symbol())==0) 
                                          CheckForOpen();
   //else                                    CheckForClose();
//----
  }
//+------------------------------------------------------------------+

//+----------------------------------------------------------------------------+
//|      :   . aka KimIV,  http://www.kimiv.ru                   |
//+----------------------------------------------------------------------------+
//|     : 19.02.2008       >> GetTicketLastPos                           |
//|   :       -1             |
//+----------------------------------------------------------------------------+
//|  :                                                                |
//|    sy -     (""   -  ,                   |
//|                                     NULL -  )                 |
//|    op -                    (-1   -  )                  |
//|    mn - MagicNumber                (-1   -  )                    |
//+----------------------------------------------------------------------------+
int GetTicketLastPos(string sy="", int op=-1, int mn=-1) {
  datetime o;
  int      i, k=OrdersTotal(), r=-1;

  if (sy=="0") sy=Symbol();
  for (i=0; i<k; i++) {
    if (OrderSelect(i, SELECT_BY_POS, MODE_TRADES)) {
      if (OrderSymbol()==sy || sy=="") {
        if (OrderType()==OP_BUY || OrderType()==OP_SELL) {
          if (op<0 || OrderType()==op) {
            if (mn<0 || OrderMagicNumber()==mn) {
              if (o<OrderOpenTime()) {
                o=OrderOpenTime();
                r=OrderTicket();
              }
            }
          }
        }
      }
    }
  }
  return(r);
}